<?php
// $Id: contact_save.admin.inc,v 1.8 2010/03/24 22:46:00 joelstein Exp $

/**
 * Displays contact form submissions.
 */
function contact_save_submissions(&$form_state) {
  
  // Get all available categories
  $categories = array(0 => t('All categories'));
  $result = db_query("SELECT DISTINCT(cid), category FROM {contact_save} ORDER BY category");
  while ($row = db_fetch_object($result)) {
    $categories[$row->cid] = $row->category;
  }
  
  // Get cid from session (but only if it matches an available category)
  $cid = (isset($_SESSION['contact_save_cid']) and isset($categories[$_SESSION['contact_save_cid']]))
    ? $_SESSION['contact_save_cid']
    : 0;
  
  // Category filter
  $form['cid'] = array(
    '#type' => 'select',
    '#title' => t('Filter by Category'),
    '#options' => $categories,
    '#default_value' => $cid,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Filter'),
  );
  
  // Build sql query
  $sql = "SELECT cs.id, cs.created, cs.name, cs.uid, cs.email, cs.subject, cs.cid, cs.replied, c.category, u.name AS u_name
    FROM {contact_save} cs
    LEFT JOIN {contact} c ON cs.cid = c.cid
    LEFT JOIN {users} u ON cs.uid = u.uid";
  $limit = 20;
  $header = array(
    array('data' => t('Date'), 'field' => 'cs.created', 'sort' => 'desc'),
    array('data' => t('Name'), 'field' => 'cs.name'),
    array('data' => t('E-mail'), 'field' => 'cs.email'),
    array('data' => t('Subject'), 'field' => 'cs.subject'),
    array('data' => t('Category'), 'field' => 'c.category'),
    array('data' => t('Replied'), 'field' => 'cs.replied'),
    array('data' => t('Actions'), 'colspan' => 2),
  );
  $tablesort = tablesort_sql($header);
  
  // Apply filter (if necessary)
  if ($cid != 0) {
    $sql .= " WHERE cs.cid = %d";
    $result = pager_query($sql . $tablesort, $limit, 0, NULL, $cid);
  } else {
    $result = pager_query($sql . $tablesort, $limit);
  }
  
  // Populate table rows
  $rows = array();
  while ($submission = db_fetch_object($result)) {
    $rows[] = array(
      _contact_save_format_field('created-date', $submission),
      _contact_save_format_field('name', $submission),
      _contact_save_format_field('email', $submission),
      _contact_save_format_field('subject', $submission),
      _contact_save_format_field('cid', $submission),
      _contact_save_format_field('replied-date', $submission),
      l('view', 'admin/build/contact/submissions/' . $submission->id),
      l('delete', 'admin/build/contact/submissions/' . $submission->id . '/delete', array('query' => array('destination' => 'admin/build/contact/submissions'))),
    );
  }
  
  // If it's an empty table
  if (empty($rows)) {
    $rows[] = array(array('data' => t('No contact form submissions have been made yet.'), 'colspan' => 8));
  }
  
  // Output
  $form['table'] = array('#value' => theme('table', $header, $rows));
  $form['pager'] = array('#value' => theme('pager', NULL, $limit, 0));
  return $form;
}

/**
 * Submit function submissions page.
 */
function contact_save_submissions_submit($form, &$form_state) {
  // Save filter in session
  $_SESSION['contact_save_cid'] = $form_state['values']['cid'];
}

/**
 * View a contact form submissions.
 */
function contact_save_view_submission(&$form_state, $id) {
  drupal_add_css(drupal_get_path('module', 'contact_save') . '/contact_save.css');
  
  $submission = db_fetch_object(db_query("SELECT cs.*, c.category, u.name AS u_name, r.name AS r_name
    FROM {contact_save} cs
    LEFT JOIN {contact} c ON cs.cid = c.cid
    LEFT JOIN {users} u ON cs.uid = u.uid
    LEFT JOIN {users} r ON cs.replier = r.uid
    WHERE cs.id = %d", $id));
  
  // Show message details
  $args = array(
    '!name' => _contact_save_format_field('name', $submission),
    '!email' => _contact_save_format_field('email', $submission),
    '!created' => _contact_save_format_field('created', $submission),
    '!cid' => _contact_save_format_field('cid', $submission),
    '!recipients' => _contact_save_format_field('recipients', $submission),
  );
  $output = t('Submitted by !name (!email) on !created in the category !cid, which should have been received by !recipients.', $args);
  $output = '<div class="contact-save-submitted">' . $output . '</div>';
  $output .= '<div class="contact-save-message">' . _contact_save_format_field('message', $submission) . '</div>';
  
  // Show reply details
  if ($submission->reply) {
    $output .= '<h2>Reply</h2>';
    $args = array(
      '!replier' => _contact_save_format_field('replier', $submission),
      '!replied' => _contact_save_format_field('replied', $submission),
    );
    $output .= '<div class="contact-save-replied">' . t('Submitted by !replier on !replied.', $args) . '</div>';
    $output .= '<div class="contact-save-reply">' . _contact_save_format_field('reply', $submission) . '</div>';
  }
  
  $form['top'] = array('#value' => $output);
  
  // Build the form
  $form['submission'] = array(
    '#type' => 'value',
    '#value' => $submission,
  );
  $form['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
    '#submit' => array('contact_save_delete_submit'),
  );
  
  // Only show reply form if no reply exists
  if (!$submission->reply) {
    $form['reply'] = array(
      '#type' => 'fieldset',
      '#title' => t('Reply to this message'),
      '#collapsible' => TRUE,
    );
    $form['reply']['subject'] = array(
      '#type' => 'textfield',
      '#title' => t('Subject'),
      '#default_value' => 'Re: [' . variable_get('site_name', 'Drupal') . '] ' . $submission->subject,
      '#required' => TRUE,
    );
    $args = array(
      '!name' => $submission->name,
      '!site_name' => variable_get('site_name', 'Drupal'),
    );
    $form['reply']['message'] = array(
      '#type' => 'textarea',
      '#title' => t('Message'),
      '#description' => t('This template can be modified on the ' . l('contact settings form', 'admin/build/contact/settings') . '. <em>Note</em>: The original message will be automatically included below your response.'),
      '#required' => TRUE,
      '#default_value' => t(variable_get('contact_save_reply_template', "Dear !name,\n\n\n\n-- !site_name team"), $args),
    );
    $form['reply']['copy'] = array(
      '#type' => 'checkbox',
      '#title' => t('Send yourself a copy.'),
    );
    $form['reply']['submit'] = array(
      '#type' => 'submit',
      '#value' => 'Submit',
    );
  }
  
  return $form;
}

/**
 * View page submission callback; sends the reply.
 */
function contact_save_view_submission_submit($form, &$form_state) {
  global $language;
  $values = $form_state['values'];
  $submission = $values['submission'];
  
  // Send reply
  $to = $submission->email;
  $from = variable_get('site_mail', ini_get('sendmail_from'));
  drupal_mail('contact_save', 'reply', $to, language_default(), $values, $from);
  drupal_set_message(t('Your reply has been sent.'));
  
  // Send copy
  if ($values['copy']) {
    drupal_mail('contact_save', 'copy', $from, $language, $values, $from);
  }
  
  // Save the reply
  global $user;
  $submission->reply = $values['message'];
  $submission->replied = time();
  $submission->replier = $user->uid;
  drupal_write_record('contact_save', $submission, array('id'));
}

/**
 * Delete button submit function; redirects to confirmation page.
 */
function contact_save_delete_submit($form, &$form_state) {
  $form_state['redirect'] = 'admin/build/contact/submissions/' . $form_state['values']['submission']->id . '/delete';
}

/**
 * Delete confirmation page.
 */
function contact_save_delete_submission(&$form_state, $id) {
  $submission = db_fetch_object(db_query("SELECT cs.*, u.name AS u_name
    FROM {contact_save} cs
    LEFT JOIN {users} u ON cs.uid = u.uid
    WHERE cs.id = %d", $id));
  $form['submission'] = array(
    '#type' => 'value',
    '#value' => $submission,
  );
  $message = t('Are you sure you want to delete the contact form submission from %user created %date?', array(
    '%user' => strip_tags(_contact_save_format_field('name', $submission)),
    '%date' => _contact_save_format_field('created', $submission),
  ));
  $destination = isset($_GET['destination']) ? $_GET['destination'] : 'admin/build/contact/submissions/' . $submission->id;
  return confirm_form($form, $message, $destination, t('This action cannot be undone.'), t('Delete'), t('Cancel'));
}

/**
 * Delete form submission callback.
 */
function contact_save_delete_submission_submit($form, &$form_state) {
  $submission = $form_state['values']['submission'];
  db_query("DELETE FROM {contact_save} WHERE id = %d", $submission->id);
  drupal_set_message(t('Contact form submission %id has been deleted.', array('%id' => $submission->id)));
  $form_state['redirect'] = 'admin/build/contact/submissions';
  return;
}

/**
 * Implementation of hook_mail().
 */
function contact_save_mail($key, &$message, $params) {
  switch ($key) {
    case 'reply':
    case 'copy':
      $message['subject'] = $params['subject'];
      $message['body'][] = $params['message'];
      $message['body'][] = t('Your original message:');
      $message['body'][] = $params['submission']->message;
      break;
  }
}

/**
 * Formats submission fields.
 */
function _contact_save_format_field($field, $submission) {
  switch ($field) {
    case 'created-date':
      return format_date($submission->created, 'custom', 'Y-m-d');
    case 'created':
      return format_date($submission->created, 'custom', 'F j, Y \a\t g:i a');
    case 'name':
      return $submission->u_name ? ((user_access('administer users') or user_access('access user profiles')) ? l($submission->u_name, 'user/' . $submission->uid) : $submission->u_name) : $submission->name;
    case 'email':
      return l($submission->email, 'mailto:' . $submission->email);
    case 'subject':
      return $submission->subject;
    case 'cid':
      return l($submission->category, 'admin/build/contact/edit/' . $submission->cid);
    case 'recipients':
      $links = array();
      foreach (explode(',', $submission->recipients) as $recipient) {
        $recipient = trim($recipient);
        $links[] = l($recipient, 'mailto:' . $recipient);
      }
      if (count($links) > 2) {
        $last = array_pop($links);
        return implode(', ', $links) . t(' and ') . $last;
      }
      return implode(t(' and '), $links);
    case 'message':
      return nl2br($submission->message);
    case 'replied-date':
      return $submission->replied ? format_date($submission->replied, 'custom', 'Y-m-d') : '';
    case 'replied':
      return format_date($submission->replied, 'custom', 'F j, Y \a\t g:i a');
    case 'replier':
      return l($submission->r_name, 'user/'.$submission->replier);
    case 'reply':
      return nl2br($submission->reply);
  }
}
